@@ -63,15 +63,21 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   m_editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection);
   m_editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection);
   m_editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection);
-  new KAction(i18n("Insert text"), QIconSet(loader->loadIcon("text",KIcon::Toolbar)), 0,this,SLOT(slotTextTool()),actionCollection,"Tool_text");
-  new KAction(i18n("Insert library pattern"),0,this,SLOT(slotLibraryPattern()),actionCollection,"librarypattern");
-  m_createLibraryPattern = new KAction(i18n("Create library pattern"),0,this,SLOT(slotLibraryCreateLibraryPattern()),actionCollection,"createlibrarypattern");
+  m_editMirrorRotateCopies = new KToggleAction(i18n("Mirror/rotate makes copies"), 0, 0, 0, actionCollection, "mirrorrotatecopies");
+  m_editRotate90 = new KAction(i18n("90 degrees"), 0, this, SLOT(slotEditRotate90()), actionCollection, "rotate90");
+  m_editRotate180 = new KAction(i18n("180 degrees"), 0, this, SLOT(slotEditRotate180()), actionCollection, "rotate180");
+  m_editRotate270 = new KAction(i18n("270 degrees"), 0, this, SLOT(slotEditRotate270()), actionCollection, "rotate270");
+  m_editMirrorVertical = new KAction(i18n("Vertically"), 0, this, SLOT(slotEditMirrorVertical()), actionCollection, "mirrorvertical");
+  m_editMirrorHorizontal = new KAction(i18n("Horizontally"), 0, this, SLOT(slotEditMirrorHorizontal()), actionCollection, "mirrorhorizontal");
+  new KAction(i18n("Insert text..."), QIconSet(loader->loadIcon("text",KIcon::Toolbar)), 0,this,SLOT(slotTextTool()),actionCollection,"Tool_text");
+  new KAction(i18n("Insert library pattern..."),0,this,SLOT(slotLibraryPattern()),actionCollection,"librarypattern");
+  m_createLibraryPattern = new KAction(i18n("Create library pattern..."),0,this,SLOT(slotLibraryCreateLibraryPattern()),actionCollection,"createlibrarypattern");
   KStdAction::fitToPage(this, SLOT(slotFitPage()), actionCollection);
   KStdAction::fitToWidth(this, SLOT(slotFitWidth()), actionCollection);
   KStdAction::fitToHeight(this, SLOT(slotFitHeight()), actionCollection);
   m_zoomIn = KStdAction::zoomIn(this, SLOT(slotZoomIn()), actionCollection);
   m_zoomOut = KStdAction::zoomOut(this, SLOT(slotZoomOut()), actionCollection);
-  new KAction(i18n("Extend pattern"), QIconSet(loader->loadIcon("extpattern",KIcon::User)), 0, this, SLOT(slotPatternExtend()), actionCollection, "extendpattern");
+  new KAction(i18n("Extend pattern..."), QIconSet(loader->loadIcon("extpattern",KIcon::User)), 0, this, SLOT(slotPatternExtend()), actionCollection, "extendpattern");
   new KAction(i18n("Center pattern"), QIconSet(loader->loadIcon("centerpattern",KIcon::User)), 0, this, SLOT(slotPatternCenter()), actionCollection, "centerpattern");
   new KAction(i18n("Crop canvas to pattern"), 0, this, SLOT(slotCropCanvasToPattern()), actionCollection, "croptopattern");
   m_cropToSelection = new KAction(i18n("Crop canvas to selection"), 0, this, SLOT(slotCropCanvasToSelection()), actionCollection, "croptoselection");
@@ -124,7 +130,7 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   radioAction->setExclusiveGroup("tools");
   radioAction = new KRadioAction(i18n("Select Rectangular Area"), QIconSet(loader->loadIcon("s_rect",KIcon::User)), 0, this, SLOT(slotSelectRectangle()), actionCollection, "Tool_select_rectangle");
   radioAction->setExclusiveGroup("tools");
-  radioAction = new KRadioAction(i18n("Pick colour from image"), QIconSet(loader->loadIcon("colorpicker",KIcon::User)), 0, this, SLOT(slotColorPicker()), actionCollection, "colorpicker");
+  radioAction = new KRadioAction(i18n("Pick color from image"), QIconSet(loader->loadIcon("colorpicker",KIcon::User)), 0, this, SLOT(slotColorPicker()), actionCollection, "colorpicker");
   radioAction->setExclusiveGroup("tools");
 
   radioAction = new KRadioAction(i18n("Stitches"), 0, this, SLOT(slotScaleFormatStitches()), actionCollection, "stitches");
@@ -153,7 +159,7 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   radioAction->setExclusiveGroup("StitchView");
 
   new KToggleAction(i18n("Stitch Mask"), 0, this, SLOT(slotStitchMask()), actionCollection, "StitchMask");
-  new KToggleAction(i18n("Colour Mask"), 0, this, SLOT(slotColorMask()), actionCollection, "ColourMask");
+  new KToggleAction(i18n("Color Mask"), 0, this, SLOT(slotColorMask()), actionCollection, "ColorMask");
 
   configure(true);
 
@@ -172,6 +178,11 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   connect(this,SIGNAL(selectionMade(bool)),m_cropToSelection,SLOT(setEnabled(bool)));
   connect(this,SIGNAL(selectionMade(bool)),m_backgroundToSelection,SLOT(setEnabled(bool)));
   connect(this,SIGNAL(selectionMade(bool)),m_createLibraryPattern,SLOT(setEnabled(bool)));
+  connect(this,SIGNAL(selectionMade(bool)),m_editRotate90,SLOT(setEnabled(bool)));
+  connect(this,SIGNAL(selectionMade(bool)),m_editRotate180,SLOT(setEnabled(bool)));
+  connect(this,SIGNAL(selectionMade(bool)),m_editRotate270,SLOT(setEnabled(bool)));
+  connect(this,SIGNAL(selectionMade(bool)),m_editMirrorVertical,SLOT(setEnabled(bool)));
+  connect(this,SIGNAL(selectionMade(bool)),m_editMirrorHorizontal,SLOT(setEnabled(bool)));
   connect(this,SIGNAL(contentsMoving(int,int)),this,SLOT(slotScrolling(int,int)));
   connect(&m_selectionTimer,SIGNAL(timeout()),this,SLOT(slotSelectionTimedOut()));
   emit selectionMade(false);
@@ -245,13 +256,13 @@ void EditView::slotFitBackgroundToSelection()
 
 void EditView::slotEditCut()
 {
-  m_doc->cutSelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask);
+  kapp->clipboard()->setData(new KXStitchMimeSource(m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,true)));
   clearSelection();
 }
 
 void EditView::slotEditCopy()
 {
-  m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask);
+  kapp->clipboard()->setData(new KXStitchMimeSource(m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,false)));
   clearSelection();
 }
 
@@ -262,6 +273,406 @@ void EditView::slotEditPaste()
   setMode(EditView::Paste);
 }
 
+void EditView::editRotate(int angle)
+{
+  QByteArray selection = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,!m_editMirrorRotateCopies->isChecked());
+  clearSelection();
+
+  QDataStream stream(selection, IO_ReadOnly);
+  QString scheme;
+  Q_INT32 width;
+  Q_INT32 height;
+  Q_INT8  count;
+  Q_INT8  type;
+  Q_INT32 backstitches;
+  Q_INT32 knots;
+  QPoint  start;
+  QPoint  end;
+  QColor  color;
+  FlossPalette palette;
+  PatternCanvas canvas;
+
+  struct TRANSLATION
+  {
+    Stitch::Type original;
+    Stitch::Type replace;
+  };
+
+   struct TRANSLATION rotate90Map[] =
+  {
+    { Stitch::TLQtr, Stitch::BLQtr },
+    { Stitch::TRQtr, Stitch::TLQtr },
+    { Stitch::BLQtr, Stitch::BRQtr },
+    { Stitch::BRQtr, Stitch::TRQtr },
+    { Stitch::TL3Qtr, Stitch::BL3Qtr },
+    { Stitch::TR3Qtr, Stitch::TL3Qtr },
+    { Stitch::BL3Qtr, Stitch::BR3Qtr },
+    { Stitch::BR3Qtr, Stitch::TR3Qtr },
+    { Stitch::BTHalf, Stitch::TBHalf },
+    { Stitch::TBHalf, Stitch::BTHalf },
+    { Stitch::Full, Stitch::Full }
+  };
+
+   struct TRANSLATION rotate180Map[] =
+  {
+    { Stitch::TLQtr, Stitch::BRQtr },
+    { Stitch::TRQtr, Stitch::BLQtr },
+    { Stitch::BLQtr, Stitch::TRQtr },
+    { Stitch::BRQtr, Stitch::TLQtr },
+    { Stitch::TL3Qtr, Stitch::BR3Qtr },
+    { Stitch::TR3Qtr, Stitch::BL3Qtr },
+    { Stitch::BL3Qtr, Stitch::TR3Qtr },
+    { Stitch::BR3Qtr, Stitch::TL3Qtr },
+    { Stitch::BTHalf, Stitch::TBHalf },
+    { Stitch::TBHalf, Stitch::BTHalf },
+    { Stitch::Full, Stitch::Full }
+  };
+
+   struct TRANSLATION rotate270Map[] =
+  {
+    { Stitch::TLQtr, Stitch::TRQtr },
+    { Stitch::TRQtr, Stitch::BRQtr },
+    { Stitch::BLQtr, Stitch::TLQtr },
+    { Stitch::BRQtr, Stitch::BLQtr },
+    { Stitch::TL3Qtr, Stitch::TR3Qtr },
+    { Stitch::TR3Qtr, Stitch::BR3Qtr },
+    { Stitch::BL3Qtr, Stitch::TL3Qtr },
+    { Stitch::BR3Qtr, Stitch::BL3Qtr },
+    { Stitch::BTHalf, Stitch::TBHalf },
+    { Stitch::TBHalf, Stitch::BTHalf },
+    { Stitch::Full, Stitch::Full }
+  };
+
+  stream >> scheme;
+  palette.setScheme(scheme);
+  stream >> width >> height;
+  if (angle != 180)
+  {
+    Q_INT32 tmp = height;
+    height = width;
+    width = tmp;
+  }
+  canvas.resize(width, height);
+
+  if (angle == 90)
+  {
+    for (int dx = 0 ; dx < width ; dx++)
+    {
+      for (int dy = height ; dy ; )
+      {
+        dy--;
+        stream >> count;
+        while (count--)
+        {
+          stream >> type >> color;
+          for (int i = 0 ; i < 11 ; i++)
+          {
+            if (rotate90Map[i].original == type)
+            {
+              type = rotate90Map[i].replace;
+              break;
+            }
+          }
+          canvas.addStitch(QPoint(dx,dy), (Stitch::Type)type, palette.addColor(color));
+        }
+      }
+    }
+  }
+  else if (angle == 180)
+  {
+    for (int dy = height ; dy ; )
+    {
+      dy--;
+      for (int dx = width ; dx ; )
+      {
+        dx--;
+        stream >> count;
+        while (count--)
+        {
+          stream >> type >> color;
+          for (int i = 0 ; i < 11 ; i++)
+          {
+            if (rotate180Map[i].original == type)
+            {
+              type = rotate180Map[i].replace;
+              break;
+            }
+          }
+          canvas.addStitch(QPoint(dx,dy), (Stitch::Type)type, palette.addColor(color));
+        }
+      }
+    }
+  }
+  else
+  {
+    // angle is 270
+    for (int dx = width ; dx ; )
+    {
+      dx--;
+      for (int dy = 0 ; dy < height ; dy++)
+      {
+        stream >> count;
+        while (count--)
+        {
+          stream >> type >> color;
+          for (int i = 0 ; i < 11 ; i++)
+          {
+            if (rotate270Map[i].original == type)
+            {
+              type = rotate270Map[i].replace;
+              break;
+            }
+          }
+          canvas.addStitch(QPoint(dx,dy), (Stitch::Type)type, palette.addColor(color));
+        }
+      }
+    }
+  }
+  stream >> backstitches;
+  int bl = height*2;
+  int tr = width*2;
+  for (int backstitch = 0 ; backstitch < backstitches ; backstitch++)
+  {
+    stream >> start >> end >> color;
+    if (angle == 90)
+      canvas.addBackstitch(QPoint(start.y(), bl-start.x()), QPoint(end.y(), bl-end.x()), palette.addColor(color));
+    else if (angle == 180)
+      canvas.addBackstitch(QPoint(tr-start.x(), bl-start.y()), QPoint(tr-end.x(), bl-end.y()), palette.addColor(color));
+    else
+      canvas.addBackstitch(QPoint(tr-start.y(), start.x()), QPoint(tr-end.y(), end.x()), palette.addColor(color));
+  }
+  stream >> knots;
+  for (int knot = 0 ; knot < knots ; knot++)
+  {
+    stream >> start >> color;
+    if (angle == 90)
+      canvas.addFrenchKnot(QPoint(start.y(), bl-start.x()), palette.addColor(color));
+    else if (angle == 180)
+      canvas.addFrenchKnot(QPoint(tr-start.x(), bl-start.y()), palette.addColor(color));
+    else
+      canvas.addFrenchKnot(QPoint(tr-start.y(), start.x()), palette.addColor(color));
+  }
+
+  QByteArray rotated;
+  QDataStream rotateStream(rotated, IO_WriteOnly);
+  rotateStream << scheme;
+  rotateStream << (Q_INT32)width << (Q_INT32)height;
+  for (int y = 0 ; y < height ; y++)
+  {
+    for (int x = 0 ; x < width ; x++)
+    {
+      Stitch::Queue *queue = canvas.stitchAt(QPoint(x,y));
+      if (queue)
+      {
+        rotateStream << (Q_INT8)queue->count();
+        for (int stitches = queue->count() ; stitches ; stitches--)
+        {
+          Stitch *s = queue->dequeue();
+          Floss* f = palette.flossAt(s->floss);
+          rotateStream << (Q_INT8)s->type << f->color;
+          delete s;
+        }
+      }
+      else
+        rotateStream << (Q_INT8)0;
+    }
+  }
+  rotateStream << (Q_INT32)backstitches;
+  QPtrListIterator<BackStitch>* bs = canvas.backstitches();
+  for (BackStitch* b = 0 ; (b = bs->current()) ; ++(*bs))
+  {
+    rotateStream << b->start << b->end << (palette.flossAt(b->floss))->color;
+  }
+  rotateStream << (Q_INT32)knots;
+  QPtrListIterator<Knot>* kn = canvas.knots();
+  for (Knot* k = 0 ; (k = kn->current()) ; ++(*kn))
+  {
+    rotateStream << k->pos << (palette.flossAt(k->floss))->color;
+  }
+
+  m_pasteData = new KXStitchMimeSource(rotated);
+  m_pasteDataIsLibrary = true;
+  setMode(EditView::Paste);
+}
+
+void EditView::slotEditRotate90()
+{
+  editRotate(90);
+}
+
+void EditView::slotEditRotate180()
+{
+  editRotate(180);
+}
+
+void EditView::slotEditRotate270()
+{
+  editRotate(270);
+}
+
+void EditView::editMirror(bool vertical)
+{
+  QByteArray selection = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,!m_editMirrorRotateCopies->isChecked());
+  clearSelection();
+
+  QDataStream stream(selection, IO_ReadOnly);
+  QString scheme;
+  Q_INT32 width;
+  Q_INT32 height;
+  Q_INT8  count;
+  Q_INT8  type;
+  Q_INT32 backstitches;
+  Q_INT32 knots;
+  QPoint  start;
+  QPoint  end;
+  QColor  color;
+  FlossPalette palette;
+  PatternCanvas canvas;
+
+  struct TRANSLATION
+  {
+    Stitch::Type original;
+    Stitch::Type replace;
+  };
+
+   struct TRANSLATION horizontalMap[] =
+  {
+    { Stitch::TLQtr, Stitch::TRQtr },
+    { Stitch::TRQtr, Stitch::TLQtr },
+    { Stitch::BLQtr, Stitch::BRQtr },
+    { Stitch::BRQtr, Stitch::BLQtr },
+    { Stitch::TL3Qtr, Stitch::TR3Qtr },
+    { Stitch::TR3Qtr, Stitch::TL3Qtr },
+    { Stitch::BL3Qtr, Stitch::BR3Qtr },
+    { Stitch::BR3Qtr, Stitch::BL3Qtr },
+    { Stitch::BTHalf, Stitch::TBHalf },
+    { Stitch::TBHalf, Stitch::BTHalf },
+    { Stitch::Full, Stitch::Full }
+  };
+
+   struct TRANSLATION verticalMap[] =
+  {
+    { Stitch::TLQtr, Stitch::BLQtr },
+    { Stitch::TRQtr, Stitch::BRQtr },
+    { Stitch::BLQtr, Stitch::TLQtr },
+    { Stitch::BRQtr, Stitch::TRQtr },
+    { Stitch::TL3Qtr, Stitch::BL3Qtr },
+    { Stitch::TR3Qtr, Stitch::BR3Qtr },
+    { Stitch::BL3Qtr, Stitch::TL3Qtr },
+    { Stitch::BR3Qtr, Stitch::TR3Qtr },
+    { Stitch::BTHalf, Stitch::TBHalf },
+    { Stitch::TBHalf, Stitch::BTHalf },
+    { Stitch::Full, Stitch::Full }
+  };
+
+  stream >> scheme;
+  palette.setScheme(scheme);
+  stream >> width >> height;
+  canvas.resize(width, height);
+
+  for (int dy = 0 ; dy < height ; dy++)
+  {
+    for (int dx = 0; dx < width ; dx++)
+    {
+      stream >> count;
+      while (count--)
+      {
+        stream >> type >> color;
+        if (vertical)
+        {
+          for (int i = 0 ; i < 11 ; i++)
+          {
+            if (verticalMap[i].original == type)
+            {
+              type = verticalMap[i].replace;
+              break;
+            }
+          }
+          canvas.addStitch(QPoint(dx,height-1-dy), (Stitch::Type)type, palette.addColor(color));
+        }
+        else // its horizontal
+        {
+          for (int i = 0 ; i < 11 ; i++)
+          {
+            if (horizontalMap[i].original == type)
+            {
+              type = horizontalMap[i].replace;
+              break;
+            }
+          }
+          canvas.addStitch(QPoint(width-1-dx,dy), (Stitch::Type)type, palette.addColor(color));
+        }
+      }
+    }
+  }
+  stream >> backstitches;
+  int bl = height*2;
+  int tr = width*2;
+  for (int backstitch = 0 ; backstitch < backstitches ; backstitch++)
+  {
+    stream >> start >> end >> color;
+    if (vertical)
+      canvas.addBackstitch(QPoint(start.x(), bl-start.y()), QPoint(end.x(), bl-end.y()), palette.addColor(color));
+    else
+      canvas.addBackstitch(QPoint(tr-start.x(), start.y()), QPoint(tr-end.x(), end.y()), palette.addColor(color));
+  }
+  stream >> knots;
+  for (int knot = 0 ; knot < knots ; knot++)
+  {
+    stream >> start >> color;
+    if (vertical)
+      canvas.addFrenchKnot(QPoint(start.x(), bl-start.y()), palette.addColor(color));
+    else
+      canvas.addFrenchKnot(QPoint(tr-start.x(), start.y()), palette.addColor(color));
+  }
+
+  QByteArray mirrored;
+  QDataStream mirrorStream(mirrored, IO_WriteOnly);
+  mirrorStream << scheme;
+  mirrorStream << (Q_INT32)width << (Q_INT32)height;
+  for (int y = 0 ; y < height ; y++)
+  {
+    for (int x = 0 ; x < width ; x++)
+    {
+      Stitch::Queue *queue = canvas.stitchAt(QPoint(x,y));
+      if (queue)
+      {
+        mirrorStream << (Q_INT8)queue->count();
+        for (int stitches = queue->count() ; stitches ; stitches--)
+        {
+          Stitch *s = queue->dequeue();
+          Floss* f = palette.flossAt(s->floss);
+          mirrorStream << (Q_INT8)s->type << f->color;
+          delete s;
+        }
+      }
+      else
+        mirrorStream << (Q_INT8)0;
+    }
+  }
+  QPtrListIterator<BackStitch>* bs = canvas.backstitches();
+  mirrorStream << (Q_INT32)(bs->count());
+  for (BackStitch* b = 0 ; (b = bs->current()) ; ++(*bs))
+    mirrorStream << b->start << b->end << (palette.flossAt(b->floss))->color;
+  QPtrListIterator<Knot>* kn = canvas.knots();
+  mirrorStream << (Q_INT32)(kn->count());
+  for (Knot* k = 0 ; (k = kn->current()) ; ++(*kn))
+    mirrorStream << k->pos << (palette.flossAt(k->floss))->color;
+  m_pasteData = new KXStitchMimeSource(mirrored);
+  m_pasteDataIsLibrary = true;
+  setMode(EditView::Paste);
+}
+
+void EditView::slotEditMirrorVertical()
+{
+  editMirror(true);
+}
+
+void EditView::slotEditMirrorHorizontal()
+{
+  editMirror(false);
+}
+
 void EditView::slotTextTool()
 {
   if (m_doc->palette()->currentColor() == -1)
@@ -300,7 +711,7 @@ void EditView::slotTextTool()
 
 void EditView::slotLibraryCreateLibraryPattern()
 {
-  QByteArray d = m_doc->copyArea(selectionArea());
+  QByteArray d = m_doc->copySelection(selectionArea(),m_stitchMask?m_currentStitchType:Stitch::Delete,m_colorMask,false);
   PatternLibraryDialog* dialog = new PatternLibraryDialog(this,new KXStitchMimeSource(d));
   dialog->exec();
   clearSelection();
@@ -368,11 +779,11 @@ void EditView::configure(bool init)
     action->activate();
     m_mode = Paint;
   }
-    
+
   m_cellGrouping = KXSConfig().Editor_CellGrouping;
   m_gridMajor = KXSConfig().Editor_GridMajorColor;
   m_gridMinor = KXSConfig().Editor_GridMinorColor;
-  
+
   m_showScales->setChecked(m_scales);
   m_scaleSize = KXSConfig().Editor_ScaleSize;
   if (m_scales)
@@ -418,7 +829,7 @@ int EditView::print(KPrinter* printer, QPainter* painter, int width, int height,
   QPaintDeviceMetrics pdm(printer);
   QFontMetrics fm = painter->fontMetrics();
   int dy = fm.lineSpacing();
-  
+
   if (printer->option("kde-kxstitch-instructions") == "true")
   {
     QFont f = painter->font();
@@ -453,7 +864,7 @@ int EditView::print(KPrinter* printer, QPainter* painter, int width, int height,
   }
 
   QString title = m_doc->title();
-  
+
   int reservedHeader = 2*fm.lineSpacing();
   if (title.isEmpty())
     reservedHeader = 0;
@@ -866,7 +1277,7 @@ void EditView::printContents(KPrinter *printer, QPainter *painter, int x, int y,
     // vertical lines
     for (int cx = x, dx = printable.left() ; cx < x+pageCellsWide+1 ; cx++, dx+=printCellSize)
     {
-      if (cx % 10)
+      if (cx % m_cellGrouping)
       {
         painter->setPen(op);
         painter->drawLine(dx,printable.top(),dx,printable.top()+pageCellsTall*printCellSize);
@@ -880,7 +1291,7 @@ void EditView::printContents(KPrinter *printer, QPainter *painter, int x, int y,
     // horizontal lines
     for (int cy = y, dy = printable.top() ; cy < y+pageCellsTall+1 ; cy++, dy+=printCellSize)
     {
-      if (cy % 10)
+      if (cy % m_cellGrouping)
       {
         painter->setPen(op);
         painter->drawLine(printable.left(),dy,printable.left()+pageCellsWide*printCellSize,dy);
@@ -896,10 +1307,10 @@ void EditView::printContents(KPrinter *printer, QPainter *painter, int x, int y,
   // check for how to print the stitches
   QString stitchesAs = printer->option("kde-kxstitch-stitches");
   enum ViewMode sViewMode = BWSymbols;
-  if (stitchesAs == "Color Symbols") sViewMode = ColorSymbols;
-  if (stitchesAs == "Colored Blocks") sViewMode = ColorBlocks;
-  if (stitchesAs == "Colored Blocks & Symbols") sViewMode = ColorBlocksSymbols;
-  if (stitchesAs != "None")
+  if (stitchesAs == i18n("Color Symbols")) sViewMode = ColorSymbols;
+  if (stitchesAs == i18n("Colored Blocks")) sViewMode = ColorBlocks;
+  if (stitchesAs == i18n("Colored Blocks & Symbols")) sViewMode = ColorBlocksSymbols;
+  if (stitchesAs != i18n("None"))
   {
     for (int dy = 0 ; dy < pageCellsTall ; dy++)
     {
@@ -954,10 +1365,10 @@ void EditView::printContents(KPrinter *printer, QPainter *painter, int x, int y,
 
   /** print the backstitches */
   QString backstitchesAs = printer->option("kde-kxstitch-backstitches");
-  if (backstitchesAs != "None")
+  if (backstitchesAs != i18n("None"))
   {
     enum ViewMode bsViewMode = BWLines;
-    if (backstitchesAs == "Color Lines") bsViewMode = ColorLines;
+    if (backstitchesAs == i18n("Color Lines")) bsViewMode = ColorLines;
     QPtrListIterator<BackStitch> *it = pCanvas->backstitches();
     BackStitch *pBackstitch;
     for ( ; (pBackstitch = it->current()) ; ++(*it) )
@@ -1000,7 +1411,6 @@ void EditView::contentsMousePressEvent(QMouseEvent *e)
     return;
   }
   QRect rect;
-  QImage* pasteImage;
   QPoint p = e->pos();
   if (m_selectionTimer.isActive())
   {
@@ -1082,15 +1492,67 @@ void EditView::contentsMousePressEvent(QMouseEvent *e)
     case ColorPicker: // no additional code for this tool
       break;
     case Paste:
-      pasteImage = new QImage(m_pasteData->encodedData("image/png"));
-      m_pasteImage = new QPixmap;
-      m_pasteImage->convertFromImage(pasteImage->smoothScale(pasteImage->width()/2*m_cellSize,pasteImage->height()/2*m_cellSize));
+      QByteArray array = m_pasteData->encodedData("application/kxstitch");
+      QDataStream stream(array, IO_ReadOnly);
+      QString scheme;
+      stream >> scheme;
+      Q_INT32 width, height;
+      stream >> width >> height;
+      m_pasteImage = new QPixmap(width*m_cellSize, height*m_cellSize);
+      m_pasteImage->fill();
+      QPainter painter(m_pasteImage);
+      int halfCellSize = m_cellSize/2;
+      for (int dy = 0 ; dy < height*m_cellSize ; dy+=m_cellSize)
+      {
+        for (int dx = 0 ; dx < width*m_cellSize ; dx+=m_cellSize)
+        {
+          Q_INT8 count;
+          stream >> count;
+          while (count--)
+          {
+            Q_INT8 type;
+            QColor color;
+            stream >> type >> color;
+            painter.setPen(color);
+            if (type & Stitch::TLQtr)
+              painter.drawLine(dx,dy,dx+halfCellSize,dy+halfCellSize);
+            if (type & Stitch::TRQtr)
+              painter.drawLine(dx+halfCellSize,dy+halfCellSize,dx+m_cellSize,dy);
+            if (type & Stitch::BLQtr)
+              painter.drawLine(dx,dy+m_cellSize,dx+halfCellSize,dy+halfCellSize);
+            if (type & Stitch::BRQtr)
+              painter.drawLine(dx+halfCellSize,dy+halfCellSize,dx+m_cellSize,dy+m_cellSize);
+          }
+        }
+      }
+      int backstitches;
+      stream >> backstitches;
+      while (backstitches--)
+      {
+        QPoint start, end;
+        QColor color;
+        stream >> start >> end >> color;
+        painter.setPen(color);
+        painter.drawLine(start*halfCellSize,end*halfCellSize);
+      }
+      int knots;
+      stream >> knots;
+      while (knots--)
+      {
+        QPoint start;
+        QColor color;
+        stream >> start >> color;
+        painter.setBrush(color);
+        painter.setPen(color);
+        QPoint topLeft = QPoint(start*halfCellSize)-QPoint(halfCellSize/2,halfCellSize/2);
+        QPoint bottomRight = QPoint(start*halfCellSize)+QPoint(halfCellSize/2,halfCellSize/2);
+        painter.drawPie(QRect(topLeft,bottomRight),0,5760);
+      }
+      painter.end();
       bitBlt(viewport(),contentsToViewport(cellToRect(m_start).topLeft()),m_pasteImage,QRect(0,0,m_pasteImage->width(),m_pasteImage->height()),NotXorROP);
-      delete pasteImage;
       break;
   }
   e->accept();
-//  showSelectionRect();
 }
 
 void EditView::contentsMouseMoveEvent(QMouseEvent *e)
@@ -2034,7 +2496,7 @@ void EditView::renderCell(QPainter *p, QRect rect, Stitch::Queue *pStitches,View
       if (viewMode == ColorBlocks || viewMode == ColorBlocksSymbols)
         p->setBrush(c);
       QChar s(pPalette->symbolAt(pStitch->floss));
-      if (viewMode!=BWSymbols) p->setPen(QPen(c, 2)); // this will be set from the stitch colour index
+      if (viewMode!=BWSymbols) p->setPen(QPen(c, 2)); // this will be set from the stitch color index
       switch (cst)
       {
         case Stitch::TLQtr:
