@@ -201,7 +201,7 @@ QString KXStitchDoc::title()
 }
 
 QString KXStitchDoc::author()
-{ 
+{
   return m_propertiesDialog->author();
 }
 
@@ -268,10 +268,13 @@ void KXStitchDoc::fileProperties()
       else
       {
         /** there are stitches based on the current palette
-            either refuse to change or do a conversion of the current stitches colors */
-        /** currently refuse to change, reset dialog box value to current scheme */
-        QString currentScheme = m_palette->getScheme();
-        m_propertiesDialog->setScheme(currentScheme);
+            try to change the scheme, but if this fails reset the dialog box value to current scheme
+          */
+        if (m_palette->changeScheme(m_propertiesDialog->scheme()) == false)
+        {
+          QString currentScheme = m_palette->getScheme();
+          m_propertiesDialog->setScheme(currentScheme);
+        }
       }
     }
     renderPixmap();
@@ -607,31 +610,76 @@ void KXStitchDoc::convertImage(const Magick::Image& magickImage)
   if (importDlg->exec() == QDialog::Accepted)
   {
     m_propertiesDialog->reset(); // initialise properties to users defaults
+    m_propertiesDialog->setScheme(importDlg->flossScheme());
     Magick::Image image = importDlg->modifiedImage();
     int w = image.columns();
     int h = image.rows();
-    m_propertiesDialog->setPatternSize(w,h);
-    resize(w,h);
-    int i;
     m_palette->setScheme(m_propertiesDialog->scheme());
     int pixelCount = w*h;
-    QProgressDialog progress(i18n("Converting to stitches..."), i18n("Cancel"), pixelCount, (KXStitchApp *)parent(), i18n("Progress"), true);
-    progress.setMinimumDuration(0);
-    Magick::Pixels cache(image);
-    const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
-    for (int cy = 0 ; cy < h ; cy++)
+    if (importDlg->useFractionals())
     {
-      progress.setProgress(cy*w);
-      KApplication::kApplication()->processEvents();
-      if (progress.wasCancelled()) break;
-      for (int cx = 0 ; cx < w ; cx++)
+      int pw = w/2;
+      int ph = h/2;
+      m_propertiesDialog->setPatternSize(pw, ph);
+      resize(pw, ph);
+      int i;
+      QProgressDialog progress(i18n("Converting to stitches..."), i18n("Cancel"), pixelCount, (KXStitchApp *)parent(), i18n("Progress"), true);
+      progress.setMinimumDuration(0);
+      Magick::Pixels cache(image);
+      const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
+      for (int cy = 0 ; cy < h ; cy++)
       {
-        Magick::PixelPacket packet = *pixels++;
-        if (!packet.opacity)
+        progress.setProgress(cy*w);
+        KApplication::kApplication()->processEvents();
+        if (progress.wasCancelled()) break;
+        for (int cx = 0 ; cx < w ; cx++)
         {
-          QColor c(packet.red/257, packet.green/257, packet.blue/257);
-          i = m_palette->addColor(c);
-          addStitch(cx,cy,Stitch::Full,i);
+          Magick::PixelPacket packet = *pixels++;
+          if (!packet.opacity)
+          {
+            QColor c(packet.red/257, packet.green/257, packet.blue/257);
+            i = m_palette->addColor(c);
+            if (cx % 2)
+            {
+              if (cy % 2)
+                addStitch(cx/2, cy/2, Stitch::BRQtr, i);
+              else
+                addStitch(cx/2, cy/2, Stitch::TRQtr, i);
+            }
+            else
+            {
+              if (cy % 2)
+                addStitch(cx/2, cy/2, Stitch::BLQtr, i);
+              else
+                addStitch(cx/2, cy/2, Stitch::TLQtr, i);
+            }
+          }
+        }
+      }
+    }
+    else
+    {
+      m_propertiesDialog->setPatternSize(w, h);
+      resize(w, h);
+      int i;
+      QProgressDialog progress(i18n("Converting to stitches..."), i18n("Cancel"), pixelCount, (KXStitchApp *)parent(), i18n("Progress"), true);
+      progress.setMinimumDuration(0);
+      Magick::Pixels cache(image);
+      const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
+      for (int cy = 0 ; cy < h ; cy++)
+      {
+        progress.setProgress(cy*w);
+        KApplication::kApplication()->processEvents();
+        if (progress.wasCancelled()) break;
+        for (int cx = 0 ; cx < w ; cx++)
+        {
+          Magick::PixelPacket packet = *pixels++;
+          if (!packet.opacity)
+          {
+            QColor c(packet.red/257, packet.green/257, packet.blue/257);
+            i = m_palette->addColor(c);
+            addStitch(cx,cy,Stitch::Full,i);
+          }
         }
       }
     }
@@ -794,65 +842,7 @@ void KXStitchDoc::deleteBackstitch(BackStitch *bs)
   setModified();
 }
 
-QByteArray KXStitchDoc::copyArea(QRect area)
-{
-  QByteArray a;
-  QDataStream stream(a,IO_WriteOnly);
-  stream << m_palette->getScheme();
-  stream << (Q_INT32)area.width() << (Q_INT32)area.height();
-  for (int row = area.top() ; row <= area.bottom() ; row++)
-  {
-    for (int col = area.left() ; col <= area.right() ; col++)
-    {
-      Stitch::Queue *queue = m_canvas->stitchAt(QPoint(col,row));
-      Stitch::Queue adding;
-      if (queue)
-      {
-        for (int stitches = queue->count() ; stitches ; stitches--)
-        {
-          Stitch* s = queue->dequeue();
-          adding.enqueue(new Stitch(*s));
-          queue->enqueue(s);
-        }
-      }
-      stream << (Q_INT8)adding.count();
-      for (int stitches = adding.count() ; stitches ; stitches--)
-      {
-        Stitch* s = adding.dequeue();
-        Floss* f = m_palette->flossAt(s->floss);
-        stream << (Q_INT8)s->type << f->color;
-        delete s;
-      }
-    }
-  }
-  int xs = area.left()*2;
-  int ys = area.top()*2;
-  int ws = area.width()*2+2;
-  int hs = area.height()*2+2;
-  QPoint tl(xs,ys);
-  QRect snapRect(xs,ys,ws,hs);
-  QPtrListIterator<BackStitch>* bs = m_canvas->backstitches();
-  QPtrList<BackStitch> addBS;
-  for (BackStitch* b = 0 ; (b = bs->current()) ; ++(*bs))
-    if (snapRect.contains(b->start) && snapRect.contains(b->end))
-      addBS.append(b);
-  stream << (Q_INT32)addBS.count();
-  QPtrListIterator<BackStitch> addBSit(addBS);
-  for (BackStitch* b = 0 ; (b = addBSit.current()) ; ++addBSit)
-    stream << (b->start-tl) << (b->end-tl) << (m_palette->flossAt(b->floss))->color;
-  QPtrListIterator<Knot>* knots = m_canvas->knots();
-  QPtrList<Knot> addKnot;
-  for (Knot* k = 0 ; (k = knots->current()) ; ++(*knots))
-    if (snapRect.contains(k->pos))
-      addKnot.append(k);
-  stream << (Q_INT32)addKnot.count();
-  QPtrListIterator<Knot> addKnotit(addKnot);
-  for (Knot* k = 0 ; (k = addKnotit.current()) ; ++addKnotit)
-    stream << (k->pos-tl) << (m_palette->flossAt(k->floss))->color;
-  return a;
-}
-
-void KXStitchDoc::copyToSelection(QRect area, Stitch::Type stitchMask, bool colorMask, bool cut)
+QByteArray KXStitchDoc::copySelection(QRect area, Stitch::Type stitchMask, bool colorMask, bool cut)
 {
   QByteArray a;
   QDataStream stream(a,IO_WriteOnly);
@@ -870,7 +860,7 @@ void KXStitchDoc::copyToSelection(QRect area, Stitch::Type stitchMask, bool colo
         {
           bool add = true;
           Stitch* s = queue->dequeue();
-          if (colorMask && s->floss != m_palette->currentColor()) add = false; //skip this one as it isn't of the specified colour
+          if (colorMask && s->floss != m_palette->currentColor()) add = false; //skip this one as it isn't of the specified color
           if (stitchMask && s->type != stitchMask) add = false; //skip this one as it isn't of the specified type
           if (add)
           {
@@ -932,18 +922,8 @@ void KXStitchDoc::copyToSelection(QRect area, Stitch::Type stitchMask, bool colo
     if (cut)
       m_canvas->deleteFrenchKnot(k->pos);
   }
-  kapp->clipboard()->setData(new KXStitchMimeSource(a));
   renderPixmap();
-}
-
-void KXStitchDoc::cutSelection(QRect area, Stitch::Type stitchMask, bool colorMask)
-{
-  copyToSelection(area, stitchMask, colorMask, true);
-}
-
-void KXStitchDoc::copySelection(QRect area, Stitch::Type stitchMask, bool colorMask)
-{
-  copyToSelection(area, stitchMask, colorMask, false);
+  return a;
 }
 
 void KXStitchDoc::pasteSelection(QPoint point, QByteArray data, bool merge)
@@ -1018,7 +998,7 @@ UsageMap KXStitchDoc::flossUsage()
     }
     stitchLength /= 100.0;       // convert to m
     backstitchLength /= 100.0;   // convert to m
-    
+
     usageMap[it.key()].stitchLength = stitchLength;
     usageMap[it.key()].backstitchLength = backstitchLength;
   }
