@@ -181,13 +181,13 @@ PaletteView::PaletteView(KXStitchDoc* doc,QWidget *parent)
 {
   KIconLoader* loader = KGlobal::iconLoader();
   KActionCollection* actionCollection = ((KXStitchApp*)topLevelWidget())->actionCollection();
-  new KAction(i18n("Add colour to palette"), QIconSet(loader->loadIcon("appearance",KIcon::User)), 0, this, SLOT(slotAddColor()), actionCollection, "addcolor");
-  new KAction(i18n("Remove colors from palette"), 0, this, SLOT(slotRemoveColors()), actionCollection, "removecolors");
-  m_savePalette = new KAction(i18n("Save the current palette"), 0, this, SLOT(slotSavePalette()), actionCollection, "savepalette");
-  m_loadPalette = new KAction(i18n("Load a palette"), 0, this, SLOT(slotLoadPalette()), actionCollection, "loadpalette");
-  m_createPalette = new KAction(i18n("Create a palette"), 0, this, SLOT(slotCreatePalette()), actionCollection, "createpalette");
+  new KAction(i18n("Add color to palette..."), QIconSet(loader->loadIcon("appearance",KIcon::User)), 0, this, SLOT(slotAddColor()), actionCollection, "addcolor");
+  new KAction(i18n("Remove colors from palette..."), 0, this, SLOT(slotRemoveColors()), actionCollection, "removecolors");
+  m_savePalette = new KAction(i18n("Save the current palette..."), 0, this, SLOT(slotSavePalette()), actionCollection, "savepalette");
+  m_loadPalette = new KAction(i18n("Load a palette..."), 0, this, SLOT(slotLoadPalette()), actionCollection, "loadpalette");
+  m_createPalette = new KAction(i18n("Create a palette..."), 0, this, SLOT(slotCreatePalette()), actionCollection, "createpalette");
   new KToggleAction(i18n("Show symbols"), 0, this, SLOT(slotShowSymbols()), actionCollection, "showsymbols");
-  new KAction(i18n("Clear unused colors..."), 0, this, SLOT(slotClearUnused()), actionCollection, "clearunused");
+  new KAction(i18n("Clear unused colors"), 0, this, SLOT(slotClearUnused()), actionCollection, "clearunused");
   new KAction(i18n("Swap colors"), 0, this, SLOT(slotPrepareToSwap()), actionCollection, "swapcolors");
   new KAction(i18n("Replace color"), 0, this, SLOT(slotPrepareToReplace()), actionCollection, "replacecolor");
 }
