@@ -389,18 +389,52 @@ bool PatternCanvas::addStitch(QPoint c, Stitch::Type t, int i)
   {
     /** no stitch queue currently exists for this location */
     pStitches = new Stitch::Queue();
-    m_pCanvas->insert(id, pStitches); 				// insert a new stitch queue
+    m_pCanvas->insert(id, pStitches);           // insert a new stitch queue
   }
 
-  uint nStitches = pStitches->count();			// get the number of stitches in the queue (may be none)
-  pStitches->enqueue(new Stitch(t, i));			// add the new stitch to the end of the queue
+  uint nStitches = pStitches->count();          // get the number of stitches in the queue (may be none)
+
+  uint n = nStitches;
+  while (n--)
+  {
+    Stitch *pStitch = pStitches->dequeue();
+    if (pStitch->floss == i)
+    {
+      t = (Stitch::Type)(t | pStitch->type);
+    }
+    pStitches->enqueue(pStitch);
+  }
+
+  switch (t)
+  {
+    case 3:
+      pStitches->enqueue(new Stitch(Stitch::TLQtr, i));
+      pStitches->enqueue(new Stitch(Stitch::TRQtr, i));
+      break;
+    case 5:
+      pStitches->enqueue(new Stitch(Stitch::TLQtr, i));
+      pStitches->enqueue(new Stitch(Stitch::BLQtr, i));
+      break;
+    case 10:
+      pStitches->enqueue(new Stitch(Stitch::TRQtr, i));
+      pStitches->enqueue(new Stitch(Stitch::BRQtr, i));
+      break;
+    case 12:
+      pStitches->enqueue(new Stitch(Stitch::BLQtr, i));
+      pStitches->enqueue(new Stitch(Stitch::BRQtr, i));
+      break;
+    default:
+      // other values are acceptable as is
+      pStitches->enqueue(new Stitch(t, i));
+      break;
+  }
 
   /** iterate the queue of existing stitches for any that have been overwritten by the new stitch */
-  while (nStitches--)											// while there are existing stitches
+  while (nStitches--)                     // while there are existing stitches
   {
-    Stitch *pStitch = pStitches->dequeue();	// get the stitch at the head of the queue
-    Stitch::Type cst = pStitch->type;					// and find its type
-    int cfi = pStitch->floss;								// and colour
+    Stitch *pStitch = pStitches->dequeue(); // get the stitch at the head of the queue
+    Stitch::Type cst = pStitch->type;         // and find its type
+    int cfi = pStitch->floss;               // and color
     Stitch::Type interferenceMask = (Stitch::Type)(cst & t);
                                             // interferenceMask now contains a mask of which bits are affected by new stitch
     if (interferenceMask)
@@ -439,13 +473,13 @@ bool PatternCanvas::addStitch(QPoint c, Stitch::Type t, int i)
       if (changeMask)
       {
                                             // Check there is something left of the original stitch
-        pStitch->type = changeMask;					// and change stitch type to the changeMask value
-        pStitches->enqueue(pStitch);				// and then add it back to the queue
+        pStitch->type = changeMask;         // and change stitch type to the changeMask value
+        pStitches->enqueue(pStitch);        // and then add it back to the queue
       }
       else
       {
                                             // if changeMask is NULL, it does not get requeued, effectively deleting it from the pattern
-        delete pStitch;											// delete the Stitch as it is no longer required
+        delete pStitch;                     // delete the Stitch as it is no longer required
       }
     }
     else
@@ -466,18 +500,18 @@ int PatternCanvas::findColor(QPoint cell, Stitch::Type stitchType)
     Stitch::Queue *pStitches = m_pCanvas->at(id); // get the pointer to any existing queue of stitches for this location
     if (pStitches)
     {
-      uint nStitches = pStitches->count();			// get the number of stitches in the queue (may be none)
+      uint nStitches = pStitches->count();      // get the number of stitches in the queue (may be none)
       /** iterate the queue of existing stitches */
-      while (nStitches--)											// while there are existing stitches
+      while (nStitches--)                     // while there are existing stitches
       {
-        Stitch *pStitch = pStitches->dequeue();	// get the stitch at the head of the queue
-        Stitch::Type cst = pStitch->type;					// and find its type
-        int cfi = pStitch->floss;								// and colour
+        Stitch *pStitch = pStitches->dequeue(); // get the stitch at the head of the queue
+        Stitch::Type cst = pStitch->type;         // and find its type
+        int cfi = pStitch->floss;               // and color
         if (colorIndex == -1)
-          colorIndex = cfi;											// set the colorIndex to the first color found
+          colorIndex = cfi;                     // set the colorIndex to the first color found
         if (cst & stitchType)
-          colorIndex = cfi;											// change the colorIndex if the current stitch type found
-        pStitches->enqueue(pStitch);						// put the stitch back in the queue
+          colorIndex = cfi;                     // change the colorIndex if the current stitch type found
+        pStitches->enqueue(pStitch);            // put the stitch back in the queue
       }
     }
   }
@@ -604,7 +638,7 @@ void PatternCanvas::writeCanvas(QDataStream &s)
   sq = m_pBackstitch->count();
   s << (Q_INT32)sq;
   for (BackStitch* bs = m_pBackstitch->first() ; bs ; bs = m_pBackstitch->next())
-    s << bs->start << bs->end << (Q_INT16)(bs->floss);		// QPoint, QPoint, Q_INT16
+    s << bs->start << bs->end << (Q_INT16)(bs->floss);    // QPoint, QPoint, Q_INT16
 }
 
 bool PatternCanvas::readPCStitch5Canvas(QDataStream& s)
@@ -794,7 +828,7 @@ UsageMap PatternCanvas::calculateUsage()
 {
   // Calculate size of stitch cell relative to pattern properties
   // and calculate relative size of a half stitch and quarter stitch
-  // Iterate canvas cells, iterating stitches in cell totalling the quantity of 
+  // Iterate canvas cells, iterating stitches in cell totalling the quantity of
   // thread required
   // Required ClothCount, ClothCountUnits
   UsageMap usageMap;
